import discord
from discord import ui
from redbot.core import Config, commands


class Forums(commands.Cog):
    """Custom cog intended for use on the Galaxy discord server.
    Developed by SeaswimmerTheFsh."""

    def __init__(self, bot):
        self.bot = bot
        self.config = Config.get_conf(self, identifier=2352711325)
        self.config.register_guild(
            request_roles=[],
            forum_channel=None,
            forum_tag=None
        )

    @commands.command()
    @commands.guild_only()
    async def resolved(self, ctx: commands.Context, *, reason: str = None):
        """Marks a thread as resolved."""
        channel_id = await self.config.guild(ctx.guild).forum_channel()
        tag = await self.config.guild(ctx.guild).forum_tag()
        request_role_ids = await self.config.guild(ctx.guild).request_roles()
        if channel_id is None or tag is None or request_role_ids is None:
            await ctx.reply(f"Configuration not set properly! Please set configuration options with `{ctx.prefix}resolvedset`.")
            return
        if isinstance(ctx.channel, discord.Thread) and ctx.channel.parent_id == channel_id:
            request_roles = [ctx.guild.get_role(role_id) for role_id in request_role_ids]
            match = any(role in ctx.author.roles for role in request_roles)
            passed_info = {
                "ctx": ctx
            }
            if match and reason:
                passed_info.update({"reason": reason})
            if match or ctx.author.id == ctx.channel.owner.id:
                msg = await ctx.send("Are you sure you'd like to mark this thread as resolved?")
                passed_info.update({"msg": msg})
                await msg.edit(view=self.ResolvedButtons(timeout=180, passed_info=passed_info))
        else:
            await ctx.message.add_reaction("❌")
            await ctx.message.delete(delay=5)

    class ResolvedButtons(ui.View):
        def __init__(self, timeout, passed_info: dict):
            super().__init__()
            self.timeout = timeout
            self.ctx: commands.Context = passed_info['ctx']
            self.msg: discord.Message = passed_info['msg']
            if 'reason' in passed_info:
                self.reason: str = passed_info['reason']
            else:
                self.reason = False
            self.config = Config.get_conf(None, cog_name='Forums', identifier=2352711325)

        @ui.button(label="Yes", style=discord.ButtonStyle.success, emoji="✅")
        async def resolved_button_yes(self, interaction: discord.Interaction, button: ui.Button): # pylint: disable=unused-argument
            request_role_ids = await self.config.guild(interaction.guild).request_roles()
            request_roles = [interaction.guild.get_role(role_id) for role_id in request_role_ids]
            match = any(role in interaction.user.roles for role in request_roles)
            if match or interaction.user.id == interaction.channel.owner.id:
                await interaction.response.defer()
                if self.reason:
                    response_reason = f"Thread closed by {interaction.user.mention} with reason: {self.reason}"
                    reason = f"Thread closed by {interaction.user.name} ({interaction.user.id}) with reason: {self.reason}"
                else:
                    response_reason = f"Thread closed by {interaction.user.mention}"
                    reason = f"Thread closed by {interaction.user.name} ({interaction.user.id})"
                await self.msg.edit(content=response_reason, view=None)
                await self.ctx.message.delete()
                tag = interaction.channel.parent.get_tag(await self.config.guild(interaction.guild).forum_tag())
                if tag in interaction.channel.applied_tags:
                    await interaction.channel.edit(locked=True, archived=True, reason=reason)
                else:
                    await interaction.channel.edit(locked=True, archived=True, applied_tags=interaction.channel.applied_tags + [tag], reason=reason)
            else:
                await interaction.response.send_message(content="You cannot close this thread!", ephemeral=True)

        @ui.button(label="No", style=discord.ButtonStyle.danger, emoji="✖️")
        async def resolved_button_no(self, interaction: discord.Interaction, button: ui.Button): # pylint: disable=unused-argument
            request_role_ids = await self.config.guild(interaction.guild).request_roles()
            request_roles = [interaction.guild.get_role(role_id) for role_id in request_role_ids]
            match = any(role in interaction.user.roles for role in request_roles)
            if match or interaction.user.id == interaction.channel.owner.id:
                await interaction.response.defer()
                await self.msg.delete()
                await self.ctx.message.delete()
            else:
                await interaction.response.send_message(content="You cannot close this thread!", ephemeral=True)

    @commands.group(name='resolvedset', autohelp=True, aliases=['resolvedconfig'])
    @commands.guild_only()
    @commands.admin()
    async def resolvedset(self, ctx: commands.Context):
        """Manages the configuration for the [p]resolved command."""

    @resolvedset.command(name='show', aliases=['settings'])
    async def resolvedset_show(self, ctx: commands.Context):
        """Shows the current cog configuration."""
        channel_id = await self.config.guild(ctx.guild).forum_channel()
        tag = await self.config.guild(ctx.guild).forum_tag()
        request_role_ids = await self.config.guild(ctx.guild).request_roles()
        split_content = ctx.message.content.split()
        command = ' '.join(split_content[:1])
        already_in_list = []
        for role_id in request_role_ids:
            role_obj = ctx.guild.get_role(role_id)
            if role_obj:
                already_in_list.append(role_obj.mention)
        if already_in_list:
            roles_list = "**Allowed Roles**:\n" + "\n".join(already_in_list)
        else:
            roles_list = f"No roles are currently in the allowed roles list.\n- Use `{command} add` to add some."
        tag_str = None
        if channel_id is not None:
            channel_obj = ctx.guild.get_channel(channel_id)
            if channel_obj is None:
                channel = f"**Channel**: {channel_id}\n- ⚠️ This channel cannot be found in this guild. Is this the correct ID?\n\n"
            else:
                channel = f"**Channel**: {channel_obj.mention}\n\n"
                if tag is not None:
                    tag_obj = channel_obj.get_tag(tag)
                    if tag_obj is None:
                        tag_str = f"**Tag**: {tag}\n- ⚠️ This tag cannot be found in the set forums channel. Is this the correct ID?\n\n"
                    else:
                        tag_str = f"**Tag**: {tag_obj.emoji} {tag_obj.name}\n\n"
        else:
            channel = f"**Channel**: Not set!\n- Use `{command} channel` to set the forums channel.\n\n"
        if tag_str is None:
            tag_str = f"**Tag**: Not set!\n- Use `{command} tag` to set the tag.\n\n"
        embed = discord.Embed(title="Cog Settings", color=await self.bot.get_embed_color(None), description=channel + tag_str + roles_list)
        await ctx.reply(embed=embed)

    @resolvedset.group(name='role', autohelp=True, aliases=['roles'])
    async def resolvedset_role(self, ctx: commands.Context):
        """Manages the allowed roles list."""

    @resolvedset_role.command(name='add')
    async def resolvedset_role_add(self, ctx: commands.Context, role: discord.Role = None):
        """Adds roles to the allowed roles list."""
        current_list = await self.config.guild(ctx.guild).request_roles()
        if role:
            if role.id in current_list:
                await ctx.send("This role is already in the allowed roles list!")
            else:
                current_list.append(role.id)
                await self.config.guild(ctx.guild).request_roles.set(current_list)
                await ctx.send(f"{role.mention} has been added to the allowed roles list.", allowed_mentions = discord.AllowedMentions(roles=False))
        else:
            await ctx.send("Please provide a valid role.")

    @resolvedset_role.command(name='remove')
    async def resolvedset_role_remove(self, ctx: commands.Context, role: discord.Role = None):
        """Removes roles from the allowed roles list."""
        current_list = await self.config.guild(ctx.guild).request_roles()
        if role.id in current_list:
            current_list.remove(role.id)
            await self.config.guild(ctx.guild).request_roles.set(current_list)
            await ctx.send(f"{role.mention} has been removed from the allowed roles list.", allowed_mentions = discord.AllowedMentions(roles=False))
        else:
            await ctx.send("Please provide a valid role that exists in the allowed roles list.")

    def create_select_options(self, ctx: commands.Context, data):
        options = []
        for tag in data:
            emoji = ctx.guild.get_emoji(tag.emoji.id) if tag.emoji.id else str(tag.emoji.name)
            options.append(discord.SelectOption(label=tag.name, emoji=emoji, description="", value=tag.id))
        return options

    @resolvedset.command(name="channel")
    async def resolvedset_channel(self, ctx: commands.Context, channel: discord.abc.GuildChannel):
        """Sets the channel used by the [p]resolved command."""
        if isinstance(channel, discord.ForumChannel):
            await self.config.guild(ctx.guild).forum_channel.set(channel.id)
            await ctx.reply(f"Forum channel has been set to {channel.mention}.")
        else:
            await ctx.reply(f"{channel.mention} is not a forums channel!")

    @resolvedset.command(name="tag")
    async def resolvedset_tag(self, ctx: commands.Context):
        """Sets the tag used by the [p]resolved command."""
        channel: discord.ForumChannel = ctx.guild.get_channel(await self.config.guild(ctx.guild).forum_channel())
        if channel is not None:
            options = self.create_select_options(ctx, channel.available_tags)
            msg = await ctx.reply("Select a forum tag below.")
            await msg.edit(view=SelectView(msg, options))
        else:
            await ctx.reply("Configuration error! Channel does not exist.")

class Select(ui.Select):
    def __init__(self, message, options):
        self.message = message
        super().__init__(placeholder="Select an option", max_values=1, min_values=1, options=options)

    async def callback(self, interaction: discord.Interaction):
        msg: discord.Message = self.message
        config = Config.get_conf(None, cog_name='Forums', identifier=2352711325)
        await config.guild(msg.guild).forum_tag.set(int(self.values[0]))
        channel: discord.ForumChannel = msg.guild.get_channel(await config.guild(msg.guild).forum_channel())
        tag = channel.get_tag(int(self.values[0]))
        await msg.edit(content=f"Set resolved tag to {tag.emoji} {tag.name}", view=None)
        await interaction.response.defer()

class SelectView(ui.View):
    def __init__(self, message, options, *, timeout=180):
        super().__init__(timeout=timeout)
        self.add_item(Select(message, options))
